@@ -4,8 +4,6 @@
 
 #if defined(EXTUNIX_HAVE_ATFILE)
 
-#include "common.h"
-
 /* otherlibs/unix/cst2constr.h */
 extern value cst_to_constr(int n, int * tbl, int size, int deflt);
 
@@ -64,15 +62,14 @@ CAMLprim value caml_extunix_fstatat(value v_dirfd, value v_name, value v_flags)
   CAMLparam3(v_dirfd, v_name, v_flags);
   int ret;
   struct stat buf;
-  char* p = caml_stat_alloc(caml_string_length(v_name) + 1);
+  char* p = strdup(String_val(v_name));
   int flags = caml_convert_flag_list(v_flags, at_flags_table);
   flags &= (AT_SYMLINK_NOFOLLOW | AT_NO_AUTOMOUNT); /* only allowed flags here */
 
-  strcpy(p, String_val(v_name));
   caml_enter_blocking_section();
   ret = fstatat(Int_val(v_dirfd), p, &buf, flags);
   caml_leave_blocking_section();
-  caml_stat_free(p);
+  free(p);
   if (ret != 0) uerror("fstatat", v_name);
   if (buf.st_size > Max_long && (buf.st_mode & S_IFMT) == S_IFREG)
     unix_error(EOVERFLOW, "fstatat", v_name);
@@ -82,16 +79,15 @@ CAMLprim value caml_extunix_fstatat(value v_dirfd, value v_name, value v_flags)
 CAMLprim value caml_extunix_unlinkat(value v_dirfd, value v_name, value v_flags)
 {
   CAMLparam3(v_dirfd, v_name, v_flags);
-  char* p = caml_stat_alloc(caml_string_length(v_name) + 1);
+  char* p = strdup(String_val(v_name));
   int ret = 0;
   int flags = caml_convert_flag_list(v_flags, at_flags_table);
   flags &= AT_REMOVEDIR;  /* only allowed flag here */
 
-  strcpy(p, String_val(v_name));
   caml_enter_blocking_section();
   ret = unlinkat(Int_val(v_dirfd), p, flags);
   caml_leave_blocking_section();
-  caml_stat_free(p);
+  free(p);
   if (ret != 0) uerror("unlinkat", v_name);
   CAMLreturn(Val_unit);
 }
@@ -123,6 +119,28 @@ CAMLprim value caml_extunix_linkat(value v_olddirfd, value v_oldname, value v_ne
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value caml_extunix_fchownat(value v_dirfd, value v_name, value v_owner, value v_group, value v_flags)
+{
+  CAMLparam5(v_dirfd, v_name, v_owner, v_group, v_flags);
+  int ret = 0;
+  int flags = caml_convert_flag_list(v_flags, at_flags_table);
+  flags &= (AT_SYMLINK_NOFOLLOW /* | AT_EMPTY_PATH */);  /* only allowed flag here */
+  ret = fchownat(Int_val(v_dirfd), String_val(v_name), Int_val(v_owner), Int_val(v_group), flags);
+  if (ret != 0) uerror("fchownat", v_name);
+  CAMLreturn(Val_unit);
+}
+
+CAMLprim value caml_extunix_fchmodat(value v_dirfd, value v_name, value v_mode, value v_flags)
+{
+  CAMLparam4(v_dirfd, v_name, v_mode, v_flags);
+  int ret = 0;
+  int flags = caml_convert_flag_list(v_flags, at_flags_table);
+  flags &= AT_SYMLINK_NOFOLLOW;  /* only allowed flag here */
+  ret = fchmodat(Int_val(v_dirfd), String_val(v_name), Int_val(v_mode), flags);
+  if (ret != 0) uerror("fchmodat", v_name);
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value caml_extunix_symlinkat(value v_path, value v_newdirfd, value v_newname)
 {
   CAMLparam3(v_path, v_newdirfd, v_newname);
@@ -138,13 +156,12 @@ CAMLprim value caml_extunix_openat(value v_dirfd, value path, value flags, value
   char * p;
 
   cv_flags = extunix_open_flags(flags);
-  p = caml_stat_alloc(caml_string_length(path) + 1);
-  strcpy(p, String_val(path));
+  p = strdup(String_val(path));
   /* open on a named FIFO can block (PR#1533) */
   caml_enter_blocking_section();
   ret = openat(Int_val(v_dirfd), p, cv_flags, Int_val(perm));
   caml_leave_blocking_section();
-  caml_stat_free(p);
+  free(p);
   if (ret == -1) uerror("openat", path);
   CAMLreturn (Val_int(ret));
 }
@@ -185,13 +202,12 @@ CAMLprim value caml_extunix_readlinkat(value v_dirfd, value v_name)
   CAMLparam2(v_dirfd, v_name);
   CAMLlocal1(v_link);
   char* res;
-  char* p = caml_stat_alloc(caml_string_length(v_name) + 1);
-  strcpy(p, String_val(v_name));
+  char* p = strdup(String_val(v_name));
 
   caml_enter_blocking_section();
   res = readlinkat_malloc(Int_val(v_dirfd), p);
   caml_leave_blocking_section();
-  caml_stat_free(p);
+  free(p);
   if (res == NULL) uerror("readlinkat", v_name);
   v_link = caml_copy_string(res);
   free(res);
