@@ -16,7 +16,7 @@ array_of_value(value v)
   size = Wosize_val(v);
   arr = caml_stat_alloc((size + 1) * sizeof(char *));
   for (i = 0; i < size; i++)
-    arr[i] = String_val(Field(v, i));
+    arr[i] = (char *)String_val(Field(v, i));
   arr[size] = NULL;
 
   CAMLreturnT (char **, arr);
